import functools
from typing import Callable

from jeffy.validator import NoneValidator, Validator


class CommonHandlerMixin(object):
    """Common event handler decorators."""

    def common(
        self,
        validator: Validator = NoneValidator()
    ) -> Callable:
        """
        Decorator for common event.

        Automatically logs payload of events and errors with the correlation ID.

        Usage::
            >>> from jeffy.framework import get_app
            >>> app = get_app()
            >>> @app.handlers.event_logging
            ... def handler(event, context):
            ...     return event['body']['foo']
        """
        def _common(func: Callable):  # type: ignore
            @functools.wraps(func)
            def wrapper(event, context):    # type: ignore
                self.capture_correlation_id(event)
                try:
                    self.app.logger.info(event)
                    ret = func(event, context)
                    self.app.logger.info(ret)
                    return ret
                except Exception as e:
                    self.app.logger.exception(e)
                    raise e
            return wrapper
        return _common



import boto3

import botocore

from jeffy.encoding import Encoding
from jeffy.encoding.bytes import BytesEncoding
from jeffy.sdk import SdkBase


class S3(SdkBase):
    """S3 Client."""

    _resource = None

    def __init__(self, encoding: Encoding = BytesEncoding()):
        """
        Initialize S3 client.

        Parameters
        ----------
        encoding: jeffy.encoding.Encoding
        """
        super(S3, self).__init__(encoding)

    @classmethod
    def get_resource(self) -> botocore.client.BaseClient:
        """
        Get boto3 client for S3.

        Usage::
            >>> from jeffy.sdk.sns import S3
            >>> S3.get_resource().upload_file(...)
        """
        if S3._resource is None:
            S3._resource = boto3.client('s3')
        return S3._resource

    def upload_file(
        self,
        file_path: str,
        bucket_name: str,
        key: str,
        correlation_id: str = ''
    ) -> None:
        """
        Upload file to S3 bucket with correlationid.

        Usage::
            >>> from jeffy.sdk.s3 import S3
            >>> S3().upload_file(...)
        """
        if correlation_id == '':
            correlation_id = self.app.correlation_id
        self.get_resource().upload_file(
            Filename=file_path,
            Bucket=bucket_name,
            Key=key,
            ExtraArgs={
                'Metadata': {
                    self.app.correlation_attr_name: correlation_id
                }
            }
        )



from typing import Any

from jeffy import framework
from jeffy.encoding import Encoding
from jeffy.encoding.bytes import BytesEncoding
from jeffy.logging import generate_correlation_id


class SdkBase():
    """Jeffy SDK base class."""

    def __init__(self, encoding: Encoding = BytesEncoding()):
        """
        Initialize Jeffy SDK.

        Parameters
        ----------
        encoding: jeffy.encoding.Encoding
        """
        self.app = framework.get_app()
        self.encoding = encoding

    def convert_to_encoded_data(self, data: Any, correlation_id: str) -> bytes:
        """
        Convert the event payload data to encoded Jeffy format data.

        Parameters
        ----------
        data: Any
        correlation_id: str
        """
        if not isinstance(data, dict):
            data = {'message': data}
        if correlation_id != '':
            data[self.app.correlation_attr_name] = correlation_id
        elif self.app.correlation_id != '':
            data[self.app.correlation_attr_name] = self.app.correlation_id
        else:
            data[self.app.correlation_attr_name] = generate_correlation_id()
        return self.encoding.encode(data)



from typing import Dict

from jeffy.handlers.common import CommonHandlerMixin
from jeffy.handlers.rest_api import RestApiHandlerMixin
from jeffy.handlers.s3 import S3HandlerMixin
from jeffy.handlers.schedule import ScheduleHandlerMixin
from jeffy.handlers.sns import SnsHandlerMixin
from jeffy.handlers.sqs import SqsHandlerMixin
from jeffy.handlers.streams import StreamsHandlerMixin
from jeffy.logging import generate_correlation_id


class Handlers(
    CommonHandlerMixin,
    RestApiHandlerMixin,
    S3HandlerMixin,
    ScheduleHandlerMixin,
    SnsHandlerMixin,
    SqsHandlerMixin,
    StreamsHandlerMixin
):
    """Jeffy event handler decorators."""

    def __init__(self, app) -> None:    # type: ignore
        self.app = app

    def capture_correlation_id(self, payload: Dict = {}) -> str:
        """
        Automatically generates and capurures the correlation ID.

        Parameters
        ----------
        payload: dict
            Payload event with including correlation attribute
        Returns
        -------
        correlation_id : str
        """
        correlation_id = ''
        if self.app.correlation_attr_name in payload:                               # type: ignore
            correlation_id = payload[self.app.correlation_attr_name]                # type: ignore
        elif self.app.correlation_id_header in payload.get('headers', {}):          # type: ignore
            correlation_id = payload['headers'][self.app.correlation_id_header]     # type: ignore
        else:
            correlation_id = generate_correlation_id()
        self.app.logger.update_context({self.app.correlation_attr_name: correlation_id})  # type: ignore
        self.app.correlation_id = correlation_id  # type: ignore
        return correlation_id



from typing import Any, Dict

import boto3

import botocore

from jeffy.encoding import Encoding
from jeffy.encoding.json import JsonEncoding
from jeffy.sdk import SdkBase


class Kinesis(SdkBase):
    """Kinesis Client."""

    _resource = None

    def __init__(self, encoding: Encoding = JsonEncoding()):
        """
        Initialize Kinesis client.

        Parameters
        ----------
        encoding: jeffy.encoding.Encoding
        """
        super(Kinesis, self).__init__(encoding)

    @classmethod
    def get_resource(self) -> botocore.client.BaseClient:
        """
        Get boto3 client for Kinesis.

        Usage::
            >>> from jeffy.sdk.kinesis import Kinesis
            >>> Kinesis.get_resource().put_record(...)
        """
        if Kinesis._resource is None:
            Kinesis._resource = boto3.client('kinesis')
        return Kinesis._resource

    def put_record(
        self,
        stream_name: str,
        data: Any,
        partition_key: str,
        correlation_id: str = ''
    ) -> Dict:
        """
        Put recourd to Kinesis Stream with correlation_id.

        Usage::
            >>> from jeffy.sdk.kinesis import Kinesis
            >>> Kinesis().put_record(...)
        """
        return self.get_resource().put_record(
            StreamName=stream_name,
            Data=self.convert_to_encoded_data(data, correlation_id),
            PartitionKey=partition_key,
        )



import functools
from typing import Callable

from jeffy.encoding import Encoding
from jeffy.encoding.bytes import BytesEncoding
from jeffy.validator import NoneValidator, Validator


class S3HandlerMixin(object):
    """S3 event handler decorators."""

    def s3(
        self,
        encoding: Encoding = BytesEncoding(),
        validator: Validator = NoneValidator()
    ) -> Callable:
        """
        Decorator for S3 event.

        Automatically parse object body stream to Lambda.

        Usage::
            >>> from jeffy.framework import setup
            >>> from jeffy.encoding.bytes import BytesEncoding
            >>> app = get_app()
            >>> @app.handlers.s3(encoding=BytesEncoding())
            ... def handler(event, context):
            ...     return event['body']
        """
        def _s3(func: Callable) -> Callable:  # type: ignore
            @functools.wraps(func)
            def wrapper(event, context):            # type: ignore
                from jeffy.sdk.s3 import S3
                s3 = S3()
                ret = []
                for record in event['Records']:
                    bucket = record['s3']['bucket']['name']
                    key = record['s3']['object']['key']
                    try:
                        response = s3.get_resource().get_object(Bucket=bucket, Key=key)
                        self.capture_correlation_id(response.get('Metadata', {}))
                        body = encoding.decode(response['Body'].read())
                        validator.validate(body)
                        self.app.logger.info({
                            'key': key,
                            'bucket_name': bucket,
                            'metadata': response['Metadata']
                        })
                        result = func({
                            'key': key,
                            'bucket_name': bucket,
                            'body': body,
                            'metadata': response['Metadata']
                        }, context)
                        self.app.logger.info(result)
                        ret.append(result)
                    except Exception as e:
                        self.app.logger.exception(e)
                        raise e
                return ret
            return wrapper
        return _s3



from typing import Any, Dict

import boto3

import botocore

from jeffy.encoding import Encoding
from jeffy.encoding.json import JsonEncoding
from jeffy.sdk import SdkBase


class Sqs(SdkBase):
    """SQS Client."""

    _resource = None

    def __init__(self, encoding: Encoding = JsonEncoding()):
        """
        Initialize SQS client.

        Parameters
        ----------
        encoding: jeffy.encoding.Encoding
        """
        super(Sqs, self).__init__(encoding)

    @classmethod
    def get_resource(self) -> botocore.client.BaseClient:
        """
        Get boto3 client for SQS.

        Usage::
            >>> from jeffy.sdk.sqs import Sqs
            >>> Sqs.get_resource().send_message(...)
        """
        if Sqs._resource is None:
            Sqs._resource = boto3.client('sqs')
        return Sqs._resource

    def send_message(
        self,
        message: Any,
        queue_url: str,
        correlation_id: str = ''
    ) -> Dict:
        """
        Send message to SQS Queue with correlationid.

        Usage::
            >>> from jeffy.sdk.sqs import Sqs
            >>> Sqs().send_message(...)
        """
        if correlation_id == '':
            correlation_id = self.app.correlation_id
        return self.get_resource().send_message(
            QueueUrl=queue_url,
            MessageBody=self.convert_to_encoded_data(message, correlation_id).decode('utf-8')
        )



# -*- coding:utf-8 -*-

"""Serverless "Application" Framework."""

__author__ = "Serverless Operations <info@serverless-operations.com>"
__status__ = "beta"
__version__ = "1.1.0"
__date__ = "5 June 2020"



import functools
from typing import Callable

from jeffy.validator import NoneValidator, Validator


class ScheduleHandlerMixin(object):
    """Schedule event handler decorators."""

    def schedule(
        self,
        validator: Validator = NoneValidator()
    ) -> Callable:
        """
        Decorator for scheduled events.

        Usage::
            >>> from jeffy.framework import get_app
            >>> app = get_app()
            >>> @app.handlers.schedule
            ... def handler(event, context):
            ...     return event['body']['foo']
        """
        def _schedule(func: Callable) -> Callable:    # type: ignore
            @functools.wraps(func)
            def wrapper(event, context):                    # type: ignore
                validator.validate(event)
                self.capture_correlation_id(event)
                try:
                    self.app.logger.info(event)
                    ret = func(event, context)
                    self.app.logger.info(ret)
                    return ret
                except Exception as e:
                    raise e
            return wrapper
        return _schedule



import base64
import functools
from typing import Callable

from jeffy.encoding import Encoding
from jeffy.encoding.json import JsonEncoding
from jeffy.validator import NoneValidator, Validator


class StreamsHandlerMixin(object):
    """Streams event handler decorators."""

    def dynamodb_streams(
        self,
        validator: Validator = NoneValidator()
    ) -> Callable:
        """
        Decorator for Dynamodb streams event.

        Automatically divide 'Records' for making it easy to treat it
        inside main process of Lambda.

        Usage::
            >>> from jeffy.framework import get_app
            >>> app = get_app()
            >>> @app.handlers.dynamodb_streams
            ... def handler(event, context):
            ...     return event['body']['foo']
        """
        def _dynamodb_streams(func: Callable) -> Callable:  # type: ignore
            @functools.wraps(func)
            def wrapper(event, context):                    # type: ignore
                ret = []
                for record in event['Records']:
                    message = record['dynamodb']
                    validator.validate(message)
                    self.capture_correlation_id(message)
                    try:
                        self.app.logger.info(message)
                        result = func(message, context)
                        self.app.logger.info(result)
                        ret.append(result)
                    except Exception as e:
                        self.app.logger.exception(e)
                        raise e
                return ret
            return wrapper
        return _dynamodb_streams

    def kinesis_streams(
        self,
        encoding: Encoding = JsonEncoding(),
        validator: Validator = NoneValidator()
    ) -> Callable:
        """
        Decorator for Kinesis stream event.

        Automatically divide 'Records' for making it easy to treat it
        inside main process of Lambda.

        Usage::
            >>> from jeffy.framework import get_app
            >>> from jeffy.encoding.json import JsonEncoding
            >>> app = get_app()
            >>> @app.handlers.kinesis_streams(encoding=JsonEncoding())
            ... def handler(event, context):
            ...     return event['body']['foo']
        """

        def _kinesis_streams(func: Callable) -> Callable:   # type: ignore
            @functools.wraps(func)
            def wrapper(event, context):                    # type: ignore
                ret = []
                for record in event['Records']:
                    message = encoding.decode(base64.b64decode(record['kinesis']['data']))
                    validator.validate(message)
                    self.capture_correlation_id(message)
                    try:
                        self.app.logger.info(message)
                        result = func(message, context)
                        self.app.logger.info(result)
                        ret.append(result)
                    except Exception as e:
                        self.app.logger.exception(e)
                        raise e
                return ret
            return wrapper
        return _kinesis_streams



import json
from typing import Any

from jeffy.encoding import DecodeError, Encoding


class JsonEncoding(Encoding):
    """JSON encoding class."""

    def encode(self, payload: Any) -> bytes:
        """
        Encode to JSON.

        Parameters
        ----------
        payload: Any

        Returns
        -------
        payload : bytes
        """
        return json.dumps(payload).encode('utf-8')

    def decode(self, payload: bytes) -> Any:
        """
        Decode from JSON.

        Parameters
        ----------
        payload: bytes

        Returns
        -------
        payload : Any

        Raises
        ------
        jeffy.encoding.DecodeError
        """
        try:
            return json.loads(payload.decode('utf-8'))
        except (json.decoder.JSONDecodeError, TypeError) as e:
            raise DecodeError(e)



from jeffy.encoding import Encoding


class BytesEncoding(Encoding):
    """Bytes encoding class."""

    def encode(self, payload: bytes) -> bytes:
        """
        Encode to bytes.

        Parameters
        ----------
        payload: bytes

        Returns
        -------
        payload : bytes
        """
        return payload

    def decode(self, payload: bytes) -> bytes:
        """
        Decode from bytes.

        Parameters
        ----------
        payload: bytes

        Returns
        -------
        payload : Any
        """
        return payload



import functools
import json
from typing import Callable

from jeffy.encoding import DecodeError, Encoding
from jeffy.encoding.json import JsonEncoding
from jeffy.validator import NoneValidator, ValidationError, Validator


class RestApiHandlerMixin(object):
    """SNS event handler decorators."""

    def rest_api(
        self,
        encoding: Encoding = JsonEncoding(),
        validator: Validator = NoneValidator()
    ) -> Callable:
        """
        Decorator for API Gateway event.

        Automatically parse string if the 'body' can be parsed as Dictionary.
        Automatically returns 500 error if unexpected error happens.

        Parameters
        ----------
        response_headers: dict
            Response headers when 500 error get_app.

        Usage::
            >>> from jeffy.framework import setup
            >>> from jeffy.encoding.json import JsonEncoding
            >>> app = get_app()
            >>> @app.handlers.rest_api(encoding=JsonEncoding())
            ... def handler(event, context):
            ...     return event['body']['foo']
        """
        def _rest_api(func: Callable):      # type: ignore
            @functools.wraps(func)
            def wrapper(event, context):    # type: ignore
                try:
                    self.capture_correlation_id(event)
                    if event.get('body') is not None:
                        try:
                            event['body'] = encoding.decode(event.get('body', '').encode('utf-8'))
                            validator.validate(event['body'])
                        except (DecodeError, ValidationError) as e:
                            self.app.logger.exception(e)
                            return {
                                'statusCode': 400,
                                'headers': {self.app.correlation_id_header: self.app.correlation_id},
                                'body': json.dumps({
                                    'error_message': str(e)
                                })
                            }
                    self.app.logger.info(event)
                    ret = func(event, context)
                    if ret.get('headers') is not None:
                        ret['headers'].update({self.app.correlation_id_header: self.app.correlation_id})
                    self.app.logger.info(ret)
                    return ret
                except Exception as e:
                    self.app.logger.exception(e)
                    raise e
            return wrapper
        return _rest_api



from typing import Any, Dict

from jeffy.validator import ValidationError, Validator

import jsonschema


class JsonSchemaValidator(Validator):
    """JSON scheme validator."""

    def __init__(self, schema: Dict):
        """
        Initialize JsonSchemeValidator.

        Parameters
        ----------
        schema: Dict
        """
        self.schema = schema

    def validate(self, data: Any) -> None:
        """
        Validate the data by JSON schema.

        Parameters
        ----------
        data: Any

        Raises
        ------
        jeffy.validator.ValidationError
        """
        try:
            jsonschema.validate(data, self.schema)
        except jsonschema.ValidationError as e:
            raise ValidationError(e)



from abc import ABC, abstractmethod
from typing import Any


class Encoding(ABC):
    """Encoding abstract class."""

    @abstractmethod
    def encode(self, payload: Any) -> bytes:
        """
        Encode abstract method.

        Parameters
        ----------
        payload: Any

        Returns
        -------
        payload : bytes
        """
        pass

    @abstractmethod
    def decode(self, payload: bytes) -> Any:
        """
        Decode abstract method.

        Parameters
        ----------
        payload: bytes

        Returns
        -------
        payload : Any
        """
        pass


class DecodeError(Exception):
    """Decode error exception."""

    pass



import logging

from jeffy.logging import get_default_logger


class Logging(object):
    """Logging setting class."""

    def __init__(
        self,
        logger: logging.Logger = get_default_logger(),
        log_level: int = logging.INFO,
        correlation_attr_name: str = 'correlation_id'
    ):
        """
        Create new logging setting.

        Parameters
        ----------
        logger: Optional[logging.Logger]
            Logger
        log_level: int = logging.INFO
            Log level
        correlation_attr_name: str = 'correlation_id'
            The attribute name of log records for correlation
        """
        self.logger = logger
        self.logger.setLevel(log_level)
        self.correlation_attr_name = correlation_attr_name


class RestApi(object):
    """REST API setting class."""

    def __init__(
        self,
        correlation_id_header: str = 'x-jeffy-correlation-id'
    ):
        """
        Create new http api setting.

        Parameters
        ----------
        correlation_id_header: str
            The name of correlation id header
        """
        self.correlation_id_header = correlation_id_header



from abc import ABC, abstractmethod
from typing import Any


class Validator(ABC):
    """Validator abstract class."""

    @abstractmethod
    def validate(self, data: Any) -> None:
        """
        Validate abstract method.

        Parameters
        ----------
        data: Any
        """
        pass


class NoneValidator(Validator):
    """No validation class."""

    def validate(self, data: Any) -> None:
        """
        Validate method.

        Parameters
        ----------
        data: Any
        """
        pass


class ValidationError(Exception):
    """Validation error exception."""

    pass



#!/usr/bin/env python
import os
import shutil
import sys

import jeffy

from setuptools import Command, find_packages, setup

if sys.argv[-1] == "publish":
    os.system("python setup.py upload")
    sys.exit()


class UploadCommand(Command):

    user_options = []

    def initialize_options(self): pass
    def finalize_options(self):   pass

    def run(self):
        try:
            cwd = os.path.join(os.path.abspath(os.path.dirname(__file__)))
            shutil.rmtree(os.path.join(cwd, 'dist'))
        except FileNotFoundError:
            pass

        os.system("python setup.py sdist")
        os.system('twine upload dist/*')
        os.system('git tag v{0}'.format(jeffy.__version__))
        os.system('git push --tags')
        sys.exit()


setup_options = dict(
    name='jeffy',
    version=jeffy.__version__,
    description='Event and Contract Driven Serverless "Application" Framework',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Serverless Operations',
    author_email='info@serverless-operations.com',
    url='https://github.com/serverless-operations/jeffy',
    packages=find_packages(exclude=['tests*', 'handler']),
    install_requires=['jsonschema', 'boto3'],
    license="MIT License",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    keywords='aws lambda serverless faas',
    cmdclass={
        'upload': UploadCommand,
    },
)

setup(**setup_options)



from jeffy.handlers import Handlers
from jeffy.settings import (
    Logging,
    RestApi
)

app = None


class Jeffy(object):
    """Jeffy framework main class."""

    def __init__(
        self,
        logging: Logging = Logging(),
        rest_api: RestApi = RestApi()
    ):
        self.logger = logging.logger
        self.correlation_attr_name = logging.correlation_attr_name
        self.correlation_id_header = rest_api.correlation_id_header
        self.correlation_id = ''
        self.handlers = Handlers(self)


def get_app(**kwargs: dict) -> Jeffy:
    """
    Get Jeffy framework application.

    Parameters
    ----------
    logging: jeffy.settings.Logging
        Logging settings
    http_api: jeffy.settings.RestApi
        Logging settings

    Returns
    -------
    jeffy.framework.Jeffy
    """
    global app
    if app is None or len(kwargs) > 0:
        app = Jeffy(**kwargs)  # type: ignore
    return app



import json
import logging
import os
import uuid
from typing import Dict


class JsonFormatter(logging.Formatter):
    """JSON log formatter."""

    def format(self, record):   # type: ignore
        """Format log record to JSON."""
        ret = {}
        for attr, value in record.__dict__.items():
            if attr == 'created':
                value = self.formatTime(record)
            if attr == 'exc_info' and value is not None:
                value = self.formatException(value)
            if attr == 'stack_info' and value is not None:
                value = self.formatStack(value)
            if attr == 'msg':
                if not isinstance(value, dict):
                    value = str(value)
            ret[attr] = value
        return json.dumps(ret)


class ContextLogger(logging.Logger):
    """Jeffy default logger."""

    def __init__(self, *args, **kwargs):    # type: ignore
        super(ContextLogger, self).__init__(*args, **kwargs)
        self.context = {
            'aws_region': os.environ.get('AWS_REGION') or os.environ.get('AWS_DEFAULT_REGION'),
            'function_name': os.environ.get('AWS_LAMBDA_FUNCTION_NAME'),
            'function_version': os.environ.get('AWS_LAMBDA_FUNCTION_VERSION'),
            'function_memory_size': os.environ.get('AWS_LAMBDA_FUNCTION_MEMORY_SIZE'),
            'log_group_name': os.environ.get('AWS_LAMBDA_LOG_GROUP_NAME'),
            'log_stream_name': os.environ.get('AWS_LAMBDA_LOG_STREAM_NAME')
        }

    def update_context(self, attrs: Dict) -> None:
        """
        Inject addtional infomation to logs.

        Usage::
            >>> from jeffy.framework import get_app
            >>> app = get_app()
            >>> app.logger.update_context(
            >>>     {
            >>>         "attribute1": "value1",
            >>>         "attribute2": "value2",
            >>>         "attribute3": "value3"
            >>>     }
            >>> )
        """
        self.context.update(attrs)

    def makeRecord(self, *args, **kwargs):  # type: ignore
        """Make log record."""
        record = super(ContextLogger, self).makeRecord(*args, **kwargs)
        for k, v in self.context.items():
            setattr(record, k, v)
        return record


def get_default_logger(name: str = 'jeffy') -> ContextLogger:
    """
    Get the default logger of Jeffy.

    Parameters
    ----------
    name: str
        Logger name

    Returns
    -------
    jeffy.logging.ContextLogger
    """
    logger = ContextLogger(name)    # type: ignore
    h = logging.StreamHandler()
    h.setFormatter(JsonFormatter())
    logger.addHandler(h)
    return logger


def generate_correlation_id() -> str:
    """Generate correlation id."""
    return str(uuid.uuid4())



from typing import Any, Dict

import boto3

import botocore

from jeffy.encoding import Encoding
from jeffy.encoding.json import JsonEncoding
from jeffy.sdk import SdkBase


class Sns(SdkBase):
    """SNS Client."""

    _resource = None

    def __init__(self, encoding: Encoding = JsonEncoding()):
        """
        Initialize SNS client.

        Parameters
        ----------
        encoding: jeffy.encoding.Encoding
        """
        super(Sns, self).__init__(encoding)

    @classmethod
    def get_resource(self) -> botocore.client.BaseClient:
        """
        Get boto3 client for SNS.

        Usage::
            >>> from jeffy.sdk.sns import Sns
            >>> Sns.get_resource().publish(...)
        """
        if Sns._resource is None:
            Sns._resource = boto3.client('sns')
        return Sns._resource

    def publish(
        self,
        topic_arn: str,
        message: Any,
        subject: str,
        correlation_id: str = ''
    ) -> Dict:
        """
        Send message to SNS Topic with correlationid.

        Usage::
            >>> from jeffy.sdk.sns import Sns
            >>> Sns().publish(...)
        """
        if correlation_id == '':
            correlation_id = self.app.correlation_id
        return self.get_resource().publish(
            TopicArn=topic_arn,
            Message=self.convert_to_encoded_data(message, correlation_id).decode('utf-8'),
            Subject=subject)



import functools
from typing import Callable

from jeffy.encoding import Encoding
from jeffy.encoding.json import JsonEncoding
from jeffy.validator import NoneValidator, Validator


class SqsHandlerMixin(object):
    """SQS event handler decorators."""

    def sqs(
        self,
        encoding: Encoding = JsonEncoding(),
        validator: Validator = NoneValidator()
    ) -> Callable:
        """
        Decorator for sqs events.

        Automatically divide 'Records' for making it easy to treat it
        inside main process of Lambda.

        Usage::
            >>> from jeffy.framework import get_app
            >>> from jeffy.encoding.json import JsonEncoding
            >>> app = get_app()
            >>> @app.handlers.sqs(encoding=JsonEncoding())
            ... def handler(event, context):
            ...     return event['body']['foo']
        """
        def _sqs(func: Callable):           # type: ignore
            @functools.wraps(func)
            def wrapper(event, context):    # type: ignore
                ret = []
                for record in event['Records']:
                    message = encoding.decode(record['body'].encode('utf-8'))
                    validator.validate(message)
                    self.capture_correlation_id(message)
                    try:
                        self.app.logger.info(message)
                        result = func(message, context)
                        self.app.logger.info(event)
                        ret.append(result)
                    except Exception as e:
                        self.app.logger.exception(e)
                        raise e
                return ret
            return wrapper
        return _sqs
